/*
 *    snake - a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999-2000 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "snake.h"
#include <errno.h>
#include <stdlib.h>		/* malloc */
#include <string.h>		/* strcat, ... */
#include <stdio.h>		/* fopen, fclose, ... */

#define DEBUG_CONFIG

/* maximum length of command name */
#define MAXCOMMANDLEN 32
#define MAXCONFLINELEN 256

/* external data */
extern char *rootdir;
extern FILE *conf;

/* public_functions */
int parse_config ();
int register_command (char *, int (*)(char *));
int eval_command (char *, char *);

/* private functions */
static char *get_config_path (void);
static FILE *open_config (void);

/* local data */
struct t_command_table
{
  char label[MAXCOMMANDLEN];
  int (*handler) (char *);
  struct t_command_table *next;
}
 *command_table;


static char *
get_config_path ()
{
  char *confpath;

  confpath = safe_malloc (strlen (rootdir) + strlen (SNAKE_CONFIG) + 5);

  strcpy (confpath, rootdir);
  strcat (confpath, "/");
  strcat (confpath, SNAKE_CONFIG);

  return confpath;
}


static FILE *
open_config ()
{
  char *confpath;
  FILE *conf;

  confpath = get_config_path ();

  conf = fopen (confpath, "r");
  if (!conf)
    {
      fprintf (stderr, "error opening %s: %s\n", confpath, strerror (errno));
      finish ();
    }

  return conf;
}


int
parse_config ()
{
  char buf[MAXCONFLINELEN];
  char *command;
  FILE *stream;

  stream = open_config ();

  while (fgets (buf, MAXCONFLINELEN, stream))
    {

      /* this is a comment */
      if (buf[0] == '#')
	continue;

      command = strtok (buf, " \t\n");
      if (command)
	{
	  eval_command (command, strtok (NULL, "\n"));
	}

    }

  fclose (stream);

  return 1;
}

/* register a command handler in the command table */
int
register_command (char *label, int (*handler) (char *))
{
  struct t_command_table *ptr, *lptr;

  /* scan the command table */
  for (lptr = ptr = command_table; ptr; lptr = ptr, ptr = ptr->next)
    {

      /* command already registered */
      if (!strcmp (ptr->label, label))
	return 0;

    }

  /* append the new command if not already registered */
  ptr = safe_malloc (sizeof (struct t_command_table));
  strncpy (ptr->label, label, MAXCOMMANDLEN - 1);
  ptr->handler = handler;

  if (lptr)
    lptr->next = ptr;
  else
    command_table = ptr;

#ifdef DEBUG_CONFIG
  fprintf (stderr, "command '%s' registered\n", label);
#endif

  return 1;
}

/* evaluate a command with the given argument */
int
eval_command (char *label, char *arg)
{
  struct t_command_table *ptr;
  int ret;

  /* scan the command table */
  for (ptr = command_table; ptr; ptr = ptr->next)
    {

      if (!strcmp (ptr->label, label))
	{
	  ret = ptr->handler (arg);
	  fprintf (stderr, "call to handler of command '%s' returned %i\n",
		   label, ret);
	  return ret;
	}

    }

  /* unknown command */
  fprintf (stderr, "eval_command: unknown command '%s'\n", label);
  return 0;
}
