/*
 *    snake, a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999-2000 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "snake.h"
#include <stdlib.h>		/* malloc, free ... */
#include <string.h>		/* string handling funx */

/* public function */
void snake_kill (snake *);
void snake_reset (snake *);
void snake_trim (snake *);
snake *snake_make (void);

/* extern functions */

/* kill a snake */
void
snake_kill (snake * s)
{
#ifdef QUIT_ON_LAST_DEATH
  int i, onealive = 0;
#endif
  snakebody *ptr;

  /* mark it dead */
  s->died = 1;

  /* make its body EXSNAKE sprites */
  for (ptr = s->head; ptr != NULL; ptr = ptr->next)
    {
      ptr->type = EXSNAKE;
    }

  beep ();			/* beeeep */

#ifdef QUIT_ON_LAST_DEATH
  for (i = 0; zoo[i]; i++)
    if (!zoo[i]->died)
      onealive = 1;
  if (!onealive)
    {
      draw_arena ();
      updatescreen ();
      finish ();
    }
#endif

}

/* reset snake body */
void
snake_reset (snake * s)
{
  snakebody *ptr, *oldptr, *onarena;
  int x, y, dir;

  /* free space allocated for old body */
  ptr = s->head;
  while (ptr != NULL)
    {
      oldptr = ptr;
      ptr = ptr->next;
      onarena = (snakebody *) ARENAGET (oldptr->cell.y, oldptr->cell.x);
      if (onarena && onarena->self == s)
	ARENAPUT (oldptr->cell.y, oldptr->cell.x, NULL);
      free (oldptr);
    }

  /* get a random position and direction for new body */
  do
    {
      x = rand_between (0, ARENA_WIDTH - 1);
      y = rand_between (0, ARENA_HEIGHT - 1);
    }
  while (ARENAGET (y, x));
  dir = rand_between (0, 3);

  /* create the new body */
  s->head = s->tail = (snakebody *) safe_malloc (sizeof (snakebody));
  s->head->type = SNAKE;
  s->head->next = NULL;
  s->head->prev = NULL;
  s->head->cell.y = y;
  s->head->cell.x = x;

  /* assign initial length */
  s->length = 1;

  /* give it a life again (in case it was dead) */
  s->died = 0;

  /* reset the score (no Karma here...) */
  s->score = 0;

  /* welcome the reborn snake */
  message ("snake %i is back!", s->playernum);
}



/* trim a snake */
void
snake_trim (snake * s)
{
  snakebody *ptr, *oldptr;
  static sprite exsnake = {
    EXSNAKE,
    {0},
  };

  /* clean up the snake screen (?is this needed?)
   * free memory used by the body */
  ptr = s->head->next;
  while (ptr)
    {
      oldptr = ptr;
      ptr = ptr->next;
      ARENAPUT (oldptr->cell.y, oldptr->cell.x, &exsnake);
      free (oldptr);
    }

  /* reset length */
  s->length = 1;

  /* set tail */
  s->tail = s->head;

  /* link to NULL */
  s->head->next = NULL;
  s->head->prev = NULL;
}


/* allocate space for a new (1-cell-big) snake.
 * initialize snake's structure members
 * assign a random position
 */
snake *
snake_make (void)
{
  int x, y, dir;
  snake *new;			/* new snake pointer */

  do
    {
      x = rand_between (0, ARENA_WIDTH - 1);
      y = rand_between (0, ARENA_HEIGHT - 1);
    }
  while (IS_SNAKE (ARENAGET (y, x)));

  dir = rand_between (0, 3);

  /* allocate memory for the new snake */
  new = (snake *) safe_malloc (sizeof (snake));


  /* allocate space for the head/tail */
  new->head = new->tail = (snakebody *) safe_malloc (sizeof (snakebody));

  /* set head/tail types */
  new->head->type = SNAKE;

  /* set self referece */
  new->head->self = new;

  /* set head/tail references to NULL */
  new->head->next = NULL;
  new->head->prev = NULL;

  /* set snake's head and tail chains */
  new->head->cell.y = y;	/* == new->tail->y */
  new->head->cell.x = x;	/* == new->tail->x */

  /* set snake's length */
  new->length = 1;

  /* set fake snake's look (will be set by register_player()) */
  new->look = 'x';

  /* give snake a life */
  new->died = 0;

  /* mark it growing */
  new->grow = 0;

  /* set snake's direction */
  new->dir = dir;

  /* set snake's score */
  new->score = 0;

  /* return the new snake */
  return new;
}
